searchData={"items":[{"type":"module","title":"EctoFragmentExtras","doc":"`EctoFragmentExtras` is a collection of macros to enhance Ecto fragments.\n\nInstead of using Ectos `fragment` with ?-based interpolation, this module\nprovides macros (that expand into regular Ecto fragments) which allow\nnamed or inline params.","ref":"EctoFragmentExtras.html"},{"type":"macro","title":"EctoFragmentExtras.frag/1","doc":"A variant of Ecto fragment() which supports params inlined in the query string or named params depending on the number of arguments.\n\nWhen called with only a query string, this macro behaves like `inline_fragment/1`.\nWhen called with a query string and a keyword list, this macro behaves like `named_fragment/2`.\n\n```elixir\ndefmodule TestQuery do\n  import Ecto.Query\n  import EctoFragmentExtras\n\n  @default_name \"Jane Doe\"\n\n  def test_query_named do\n    query = from users in \"users\",\n            select: frag(\"coalesce(#{:a}, #{:b})\", a: users.name, b: ^@default_name)\n\n    Repo.all(query)\n  end\n\n  def test_query_inline do\n    query = from users in \"users\",\n            select: frag(\"coalesce(#{users.name}, #{^@default_name})\")\n\n    Repo.all(query)\n  end\nend\n```","ref":"EctoFragmentExtras.html#frag/1"},{"type":"macro","title":"EctoFragmentExtras.frag/2","doc":"","ref":"EctoFragmentExtras.html#frag/2"},{"type":"macro","title":"EctoFragmentExtras.inline_fragment/1","doc":"A variant of Ecto fragment() which supports params inlined in the query string.\n\nInline params are provided within the fragment query string as\n`#{}`-style string interpolation. Since this is a macro which expands to a regular\nEcto fragment, those interpolations are safe (they are no real string interpolations,\nand pose no extra SQL injection risk compared to regular fragments).\n\nSo `inline_fragment(\"coalesce(#{user.name}, #{^default})\")` will\nbe expanded to `fragment(\"coalesce(?, ?)\", user.name, ^default)` at compile-time.\n\n```elixir\ndefmodule TestQuery do\n  import Ecto.Query\n  import EctoFragmentExtras\n\n  def test_query do\n    default = \"Jane Doe\"\n    query = from users in \"users\",\n            select: inline_fragment(\"coalesce(#{users.name}, #{^default})\")\n\n    Repo.all(query)\n  end\nend\n```\n\nSAFETY WARNING: Please note that it is NOT safe to first interpolate a string which is passed in to this macro.\nUnsafe example:\n\n```elixir\ndef unsafe_query(user_name) do\n    default = \"Jane Doe\"\n    fragment_string = \"coalesce(#{user_name}, #{^default})\"\n    query = from users in \"users\",\n            select: inline_fragment(fragment_string)\n\n    Repo.all(query)\n  end\nend\n```\n\nThe code above is UNSAFE, because the interpolation is done before the macro is called and the macro\nreceives the already interpolated string. This prohibits the macro from safely escaping the query params.\nIf the `user_name` param comes from untrusted user input, this can lead to SQL injection.","ref":"EctoFragmentExtras.html#inline_fragment/1"},{"type":"macro","title":"EctoFragmentExtras.named_fragment/2","doc":"A variant of Ecto fragment() which supports named params.\n\nNameds params are provided in a keyword list as the last argument.\nThe fragments query string can reference named params using the\n`name` syntax where `name` is a key in the keyword list.\n\nSo `named_fragment(\"coalesce(a, b, a)\", a: 1, b: 2)` will\nbe expanded to `fragment(\"coalesce(?, ?, ?)\", 1, 2, 1)` at compile-time.\n\n```elixir\ndefmodule TestQuery do\n  import Ecto.Query\n  import EctoFragmentExtras\n\n  def test_query do\n    default = \"Jane Doe\"\n    query = from users in \"users\",\n            select: named_fragment(\"coalesce(a, b)\", a: users.name, b: ^default)\n\n    Repo.all(query)\n  end\nend\n```","ref":"EctoFragmentExtras.html#named_fragment/2"},{"type":"module","title":"EctoFragmentExtras.Exceptions","doc":"","ref":"EctoFragmentExtras.Exceptions.html"},{"type":"function","title":"EctoFragmentExtras.Exceptions.error!/1","doc":"","ref":"EctoFragmentExtras.Exceptions.html#error!/1"},{"type":"exception","title":"EctoFragmentExtras.Exceptions.CompileError","doc":"Raised at compilation time when the named fragment cannot be compiled.","ref":"EctoFragmentExtras.Exceptions.CompileError.html"}],"content_type":"text/markdown"}